@@ -138,8 +138,14 @@ class ChangesetEntry(object):
         3.0
     """
 
-    def __init__(self, root_path, old_path=None, new_path=None,
-                 old_oid=None, new_oid=None, op='modify'):
+    def __init__(self,
+                 root_path,
+                 old_path=None,
+                 new_path=None,
+                 old_oid=None,
+                 new_oid=None,
+                 op='modify',
+                 is_dir=False):
         """Initialize the changeset entry.
 
         Args:
@@ -160,6 +166,9 @@ class ChangesetEntry(object):
 
             op (unicode, optional):
                 The change operation.
+
+            is_dir (bool, optional):
+                Whether the changeset entry represents a directory.
         """
         self.root_path = root_path
 
@@ -174,6 +183,7 @@ class ChangesetEntry(object):
         self._new_version = None
 
         self.op = op
+        self.is_dir = is_dir
 
     @property
     def old_oid(self):
@@ -1552,11 +1562,157 @@ class ClearCaseClient(SCMClient):
         # versions. The chances of this in reality are probably pretty small.
         return self._get_branch_changeset(branch, repository_info)
 
+    def _diff_directories(self,
+                          entry,
+                          repository_info,
+                          diffx_change):
+        """Return a unified diff for a changed directory.
+
+        Args:
+            entry (ChangesetEntry):
+                The changeset entry.
+
+            repository_info (ClearCaseRepositoryInfo):
+                The repository info structure.
+
+            diffx_change (pydiffx.dom.DiffXChangeSection):
+                The DiffX DOM object for writing VersionVault diffs.
+
+        Returns:
+            list of bytes:
+            The diff between the two directory listings, for writing legacy
+            ClearCase diffs.
+        """
+        dl = []
+
+        old_content = self._get_directory_contents(entry.old_path)
+        old_tmp = make_tempfile(content=old_content)
+
+        new_content = self._get_directory_contents(entry.new_path)
+        new_tmp = make_tempfile(content=new_content)
+
+        dl = execute(['diff', '-uN', old_tmp, new_tmp],
+                     extra_ignore_errors=(1, 2),
+                     results_unicode=False,
+                     split_lines=True)
+
+        if dl:
+            dl[0] = dl[0].replace(old_tmp.encode('utf-8'),
+                                  entry.old_path.encode('utf-8'))
+            dl[1] = dl[1].replace(new_tmp.encode('utf-8'),
+                                  entry.new_path.encode('utf-8'))
+
+            if repository_info.is_legacy:
+                index_line = (
+                    b'==== %s %s ====\n'
+                    % (entry.old_oid.encode('utf-8'),
+                       entry.new_oid.encode('utf-8')))
+                dl.insert(2, index_line)
+
+        if not repository_info.is_legacy:
+            # We need oids of files to translate them to paths on reviewboard
+            # repository.
+            vob_oid = execute(['cleartool', 'describe', '-fmt', '%On',
+                               'vob:%s' % (entry.new_path or entry.old_path)])
+
+            vv_metadata = {
+                'directory-diff': 'legacy-filenames',
+                'vob': vob_oid,
+            }
+
+            if entry.op == 'create':
+                path = os.path.relpath(entry.new_path, self.root_path)
+                revision = {
+                    'new': entry.new_version,
+                }
+                vv_metadata['new'] = {
+                    'name': entry.new_name,
+                    'oid': entry.new_oid,
+                    'path': path,
+                }
+            elif entry.op == 'delete':
+                path = os.path.relpath(entry.old_path, self.root_path)
+                revision = {
+                    'old': entry.old_version,
+                }
+                vv_metadata['old'] = {
+                    'name': entry.old_name,
+                    'oid': entry.old_oid,
+                    'path': path,
+                }
+            elif entry.op in ('modify', 'move'):
+                old_path_rel = os.path.relpath(entry.old_path, self.root_path)
+                new_path_rel = os.path.relpath(entry.new_path, self.root_path)
+
+                path = {
+                    'old': old_path_rel,
+                    'new': new_path_rel,
+                }
+                revision = {
+                    'old': entry.old_version,
+                    'new': entry.new_version,
+                }
+                vv_metadata['old'] = {
+                    'name': entry.old_name,
+                    'oid': entry.old_oid,
+                    'path': old_path_rel,
+                }
+                vv_metadata['new'] = {
+                    'name': entry.new_name,
+                    'oid': entry.new_oid,
+                    'path': new_path_rel,
+                }
+            else:
+                logging.warning(
+                    'Unexpected operation "%s" for directory %s %s',
+                    entry.op, entry.old_path, entry.new_path)
+                return []
+
+            diffx_change.add_file(
+                meta={
+                    'op': entry.op,
+                    'path': path,
+                    'revision': revision,
+                    'type': 'directory',
+                    'versionvault': vv_metadata,
+                },
+                diff_type='text',
+                diff=b''.join(dl))
+
+        return dl
+
+    def _get_directory_contents(self, extended_path):
+        """Return an ls-style directory listing for a versioned directory.
+
+        Args:
+            extended_path (unicode):
+                The path of the directory to get the contents for.
+
+        Returns:
+            bytes:
+            The contents of the directory.
+        """
+        output = execute(['cleartool', 'ls', '-short', '-nxname', '-vob_only',
+                          extended_path],
+                         split_lines=True,
+                         results_unicode=False)
+
+        contents = sorted(
+            os.path.basename(absolute_path.strip())
+            for absolute_path in output
+        )
+
+        # Add one extra empty line so the data passed to diff ends in a
+        # newline.
+        contents.append(b'')
+
+        return b'\n'.join(contents)
+
     def _diff_files(self,
                     entry,
                     repository_info,
                     diffx_change):
-        """Return a unified diff for file.
+        """Return a unified diff for a changed file.
 
         Args:
             entry (ChangesetEntry):
@@ -1574,11 +1730,19 @@ class ClearCaseClient(SCMClient):
         """
         if entry.old_path:
             old_file_rel = os.path.relpath(entry.old_path, self.root_path)
+
+            if repository_info.is_legacy:
+                old_file_rel = os.path.join(repository_info.vobtag,
+                                            old_file_rel)
         else:
             old_file_rel = '/dev/null'
 
         if entry.new_path:
             new_file_rel = os.path.relpath(entry.new_path, self.root_path)
+
+            if repository_info.is_legacy:
+                new_file_rel = os.path.join(repository_info.vobtag,
+                                            new_file_rel)
         else:
             new_file_rel = '/dev/null'
 
@@ -1598,20 +1762,21 @@ class ClearCaseClient(SCMClient):
             diff_old_file = entry.old_path or make_tempfile()
             diff_new_file = entry.new_path or make_tempfile()
 
-        dl = execute(['diff', '-uN', diff_old_file, diff_new_file],
-                     extra_ignore_errors=(1, 2),
-                     results_unicode=False)
+        diff = execute(['diff', '-uN', diff_old_file, diff_new_file],
+                       extra_ignore_errors=(1, 2),
+                       results_unicode=False)
+
+        # If the input file has ^M characters at end of line, lets ignore them.
+        diff = diff.replace(b'\r\r\n', b'\r\n')
+        dl = diff.splitlines(True)
 
         # Replace temporary filenames in the diff with view-local relative
         # paths.
-        dl = dl.replace(diff_old_file.encode('utf-8'),
-                        old_file_rel.encode('utf-8'))
-        dl = dl.replace(diff_new_file.encode('utf-8'),
-                        new_file_rel.encode('utf-8'))
-
-        # If the input file has ^M characters at end of line, lets ignore them.
-        dl = dl.replace(b'\r\r\n', b'\r\n')
-        dl = dl.splitlines(True)
+        if len(dl) >= 2:
+            dl[0] = dl[0].replace(diff_old_file.encode('utf-8'),
+                                  old_file_rel.encode('utf-8'))
+            dl[1] = dl[1].replace(diff_new_file.encode('utf-8'),
+                                  new_file_rel.encode('utf-8'))
 
         # Special handling for the output of the diff tool on binary files:
         #     diff outputs "Files a and b differ"
@@ -1621,11 +1786,6 @@ class ClearCaseClient(SCMClient):
             dl[0].startswith(b'Files ') and dl[0].endswith(b' differ')):
             dl = [b'Binary f%s' % dl[0][1:]]
 
-        # We need oids of files to translate them to paths on reviewboard
-        # repository.
-        vob_oid = execute(['cleartool', 'describe', '-fmt', '%On',
-                           'vob:%s' % (entry.new_path or entry.old_path)])
-
         if dl and dl[0].startswith(b'Binary files '):
             if repository_info.is_legacy:
                 dl.insert(
@@ -1640,6 +1800,11 @@ class ClearCaseClient(SCMClient):
                                             entry.new_oid.encode('utf-8')))
 
         if not repository_info.is_legacy:
+            # We need oids of files to translate them to paths on reviewboard
+            # repository.
+            vob_oid = execute(['cleartool', 'describe', '-fmt', '%On',
+                               'vob:%s' % (entry.new_path or entry.old_path)])
+
             vv_metadata = {
                 'vob': vob_oid,
             }
@@ -1698,6 +1863,7 @@ class ClearCaseClient(SCMClient):
             else:
                 logging.warning('Unexpected operation "%s" for file %s %s',
                                 entry.op, entry.old_path, entry.new_path)
+                return []
 
             diffx_change.add_file(
                 meta={
@@ -1705,11 +1871,12 @@ class ClearCaseClient(SCMClient):
                     'path': path,
                     'op': entry.op,
                     'revision': revision,
+                    'type': 'file',
                 },
                 diff_type=diff_type,
                 diff=b''.join(dl))
 
-            return dl
+        return dl
 
     def _get_content_snapshot(self, filename):
         """Return the content of a file in a snapshot view.
@@ -1765,16 +1932,22 @@ class ClearCaseClient(SCMClient):
         })
         legacy_diff = []
 
+        logging.debug('Doing diff of changeset: %s', changeset)
+
         for entry in changeset:
             legacy_dl = []
 
-            if self.viewtype == 'snapshot' or cpath.exists(entry.new_path):
-                legacy_dl = self._diff_files(entry, repository_info,
-                                             diffx_change)
+            if entry.is_dir:
+                legacy_dl = self._diff_directories(entry, repository_info,
+                                                   diffx_change)
             else:
-                logging.error('File %s does not exist or access is denied.',
-                              entry.new_path)
-                continue
+                if self.viewtype == 'snapshot' or cpath.exists(entry.new_path):
+                    legacy_dl = self._diff_files(entry, repository_info,
+                                                 diffx_change)
+                else:
+                    logging.error('File %s does not exist or access is denied.',
+                                  entry.new_path)
+                    continue
 
             if repository_info.is_legacy and legacy_dl:
                 legacy_diff.append(b''.join(legacy_dl))
@@ -1789,6 +1962,25 @@ class ClearCaseClient(SCMClient):
             'diff': diff,
         }
 
+    def _is_dir(self, path):
+        """Return whether a given path is a directory.
+
+        Args:
+            path (unicode):
+                The path of the element to check.
+
+        Returns:
+            bool:
+            ``True`` if the given element is a directory.
+        """
+        if self.viewtype == 'dynamic' and cpath.exists(path):
+            return cpath.isdir(path)
+        else:
+            object_kind = execute(
+                ['cleartool', 'describe', '-fmt', '%m', path])
+
+            return object_kind.startswith('directory')
+
     def _process_directory_changes(self, changeset):
         """Scan through the changeset and handle directory elements.
 
@@ -1813,22 +2005,16 @@ class ClearCaseClient(SCMClient):
         directories = []
 
         for old_file, new_file in changeset:
-            if self.viewtype == 'snapshot':
-                object_kind = execute(['cleartool', 'describe', '-fmt', '%m',
-                                       new_file])
-
-                is_dir = object_kind.startswith('directory')
-            else:
-                is_dir = cpath.isdir(new_file)
-
-            if is_dir:
+            if self._is_dir(new_file):
                 directories.append((old_file, new_file))
+                files.append(ChangesetEntry(self.root_path, old_path=old_file,
+                                            new_path=new_file, is_dir=True))
             else:
                 files.append(ChangesetEntry(self.root_path, old_path=old_file,
                                             new_path=new_file))
 
         for old_dir, new_dir in directories:
-            changes = self._diff_directory(old_dir, new_dir)
+            changes = self._get_file_changes_from_directories(old_dir, new_dir)
 
             for filename, oid in changes['added']:
                 for file in files:
@@ -1836,10 +2022,11 @@ class ClearCaseClient(SCMClient):
                         file.op = 'create'
                         break
                 else:
-                    files.append(ChangesetEntry(self.root_path,
-                                                new_path=filename,
-                                                new_oid=oid,
-                                                op='create'))
+                    if not self._is_dir(filename):
+                        files.append(ChangesetEntry(self.root_path,
+                                                    new_path=filename,
+                                                    new_oid=oid,
+                                                    op='create'))
 
             for filename, oid in changes['deleted']:
                 for file in files:
@@ -1847,17 +2034,18 @@ class ClearCaseClient(SCMClient):
                         file.op = 'delete'
                         break
                 else:
-                    # The extended path we get here doesn't include the
-                    # revision of the element. While many operations can
-                    # succeed in this case, fetching the content of the file
-                    # from snapshot views does not. We therefore look at the
-                    # history of the file and get the last revision from it.
-                    filename = execute(['cleartool', 'lshistory', '-last',
-                                        '1', '-fmt', '%Xn', 'oid:%s' % oid])
-                    files.append(ChangesetEntry(self.root_path,
-                                                old_path=filename,
-                                                old_oid=oid,
-                                                op='delete'))
+                    if not self._is_dir(filename):
+                        # The extended path we get here doesn't include the
+                        # revision of the element. While many operations can
+                        # succeed in this case, fetching the content of the file
+                        # from snapshot views does not. We therefore look at the
+                        # history of the file and get the last revision from it.
+                        filename = execute(['cleartool', 'lshistory', '-last',
+                                            '1', '-fmt', '%Xn', 'oid:%s' % oid])
+                        files.append(ChangesetEntry(self.root_path,
+                                                    old_path=filename,
+                                                    old_oid=oid,
+                                                    op='delete'))
 
             for old_file, old_oid, new_file, new_oid in changes['renamed']:
                 # Just using the old filename that we get from the
@@ -1875,16 +2063,17 @@ class ClearCaseClient(SCMClient):
                         file.op = 'move'
                         break
                 else:
-                    files.append(ChangesetEntry(self.root_path,
-                                                old_path=old_path,
-                                                new_path=new_file,
-                                                old_oid=old_oid,
-                                                new_oid=new_oid,
-                                                op='move'))
+                    if not self._is_dir(new_file):
+                        files.append(ChangesetEntry(self.root_path,
+                                                    old_path=old_path,
+                                                    new_path=new_file,
+                                                    old_oid=old_oid,
+                                                    new_oid=new_oid,
+                                                    op='move'))
 
         return files
 
-    def _diff_directory(self, old_dir, new_dir):
+    def _get_file_changes_from_directories(self, old_dir, new_dir):
         """Get directory differences.
 
         This will query and parse the diff of a directory element, in order to
@@ -1938,25 +2127,38 @@ class ClearCaseClient(SCMClient):
 
             get_oid_cmd = ['cleartool', 'desc', '-fmt', '%On']
 
-            if current_mode == 'renamed to':
-                old_file = cpath.join(old_dir, _extract_filename(line))
-                old_oid = execute(get_oid_cmd + [old_file])
-                new_file = cpath.join(new_dir,
-                                      _extract_filename(diff_lines[i + 1]))
-                new_oid = execute(get_oid_cmd + [new_file])
+            try:
+                if current_mode == 'renamed to':
+                    old_file = cpath.join(old_dir, _extract_filename(line))
+                    old_oid = execute(get_oid_cmd + [old_file])
+                    new_file = cpath.join(new_dir,
+                                          _extract_filename(diff_lines[i + 1]))
+                    new_oid = execute(get_oid_cmd + [new_file])
 
-                results['renamed'].add((old_file, old_oid, new_file, new_oid))
-                i += 2
-            elif current_mode == 'added':
-                new_file = cpath.join(new_dir, _extract_filename(line))
-                oid = execute(get_oid_cmd + [new_file])
+                    results['renamed'].add(
+                        (old_file, old_oid, new_file, new_oid))
+                    i += 2
+                elif current_mode == 'added':
+                    new_file = cpath.join(new_dir, _extract_filename(line))
+                    oid = execute(get_oid_cmd + [new_file])
 
-                results['added'].add((new_file, oid))
-            elif current_mode == 'deleted':
-                old_file = cpath.join(old_dir, _extract_filename(line))
-                oid = execute(get_oid_cmd + [old_file])
+                    results['added'].add((new_file, oid))
+                elif current_mode == 'deleted':
+                    old_file = cpath.join(old_dir, _extract_filename(line))
+                    oid = execute(get_oid_cmd + [old_file])
 
-                results['deleted'].add((old_file, oid))
+                    results['deleted'].add((old_file, oid))
+            except Exception as e:
+                # It's possible that we'll get errors when trying to use
+                # get_oid_cmd in some cases, such as when a symbolic link is
+                # added or removed. In this cases, we'll just log a warning
+                # and skip them for this step. We'll still show it when the
+                # directory contents get diffed later.
+                logging.debug('Got error while processing directory changes '
+                              'from %s to %s: %s',
+                              old_dir,
+                              new_dir,
+                              e)
 
         return results
 
@@ -2159,6 +2361,9 @@ class ClearCaseRepositoryInfo(RepositoryInfo):
             self.vob_tags.update(tags)
             self.uuid_to_tags[uuid] = list(tags)
 
+        if self.is_legacy:
+            self.base_path = self.vobtag
+
     def find_server_repository_info(self, api_root):
         """Find a matching repository on the server.
 
