@@ -1,11 +1,10 @@
 """The rbt patch command."""
 
-from __future__ import print_function, unicode_literals
+from __future__ import unicode_literals
 
 import logging
 import os
 import re
-import sys
 from gettext import gettext as _, ngettext
 
 import six
@@ -32,6 +31,10 @@ class Patch(Command):
 
     name = 'patch'
     author = 'The Review Board Project'
+
+    needs_api = True
+    needs_scm_client = True
+
     args = '<review-request-id>'
     option_list = [
         Option('-c', '--commit',
@@ -160,7 +163,7 @@ class Patch(Command):
 
         # Sanity-check the arguments, making sure that the options provided
         # are compatible with each other and with the Review Board server.
-        server_supports_history = self._tool.capabilities.has_capability(
+        server_supports_history = self.capabilities.has_capability(
             'review_requests', 'supports_history')
 
         if server_supports_history:
@@ -181,7 +184,7 @@ class Patch(Command):
         # revision through the API.
         if diff_revision is None:
             try:
-                diffs = self._api_root.get_diffs(
+                diffs = self.api_root.get_diffs(
                     review_request_id=self._review_request_id,
                     only_fields='',
                     only_links='')
@@ -197,11 +200,11 @@ class Patch(Command):
         try:
             # Fetch the main diff and (unless we're squashing) any commits within.
             if squashed:
-                diff = self._api_root.get_diff(
+                diff = self.api_root.get_diff(
                     review_request_id=self._review_request_id,
                     diff_revision=diff_revision)
             else:
-                diff = self._api_root.get_diff(
+                diff = self.api_root.get_diff(
                     review_request_id=self._review_request_id,
                     diff_revision=diff_revision,
                     expand='commits')
@@ -342,25 +345,25 @@ class Patch(Command):
                     'total': total_patches,
                 })
 
-        result = self._tool.apply_patch(
+        result = self.tool.apply_patch(
             patch_file=diff_file_path,
-            base_path=self._repository_info.base_path,
+            base_path=self.repository_info.base_path,
             base_dir=base_dir,
             p=self.options.px,
             revert=revert)
 
         if result.patch_output:
-            print()
+            self.stdout.new_line()
 
             patch_output = result.patch_output.strip()
 
             if six.PY2:
-                print(patch_output)
+                self.stdout.write(patch_output)
             else:
-                sys.stdout.buffer.write(patch_output)
-                print()
+                self.stdout_bytes.write(patch_output)
+                self.stdout.new_line()
 
-            print()
+            self.stdout.new_line()
 
         if not result.applied:
             if revert:
@@ -375,29 +378,78 @@ class Patch(Command):
         if result.has_conflicts:
             if result.conflicting_files:
                 if revert:
-                    print('The patch was partially reverted, but there were '
-                          'conflicts in:')
+                    self.stdout.write('The patch was partially reverted, but '
+                                      'there were conflicts in:')
+                    self.json.add_error('The patch was partially reverted, '
+                                        'but there were conflicts.')
                 else:
-                    print('The patch was partially applied, but there were '
-                          'conflicts in:')
+                    self.stdout.write('The patch was partially applied, but '
+                                      'there were conflicts in:')
+                    self.json.add_error('The patch was partially applied, '
+                                        'but there were conflicts.')
 
-                print()
+                self.stdout.new_line()
+
+                self.json.add('conflicting_files', [])
 
                 for filename in result.conflicting_files:
-                    print('    %s' % filename)
+                    filename = filename.decode('utf-8')
+                    self.stdout.write('    %s' % filename)
+                    self.json.append('conflicting_files', filename)
 
-                print()
+                self.stdout.new_line()
             elif revert:
-                print('The patch was partially reverted, but there were '
-                      'conflicts.')
+                err = ('The patch was partially reverted, but there were '
+                       'conflicts.')
+                self.stdout.write(err)
+                self.json.add_error(err)
             else:
-                print('The patch was partially applied, but there were '
-                      'conflicts.')
+                err = ('The patch was partially applied, but there were '
+                       'conflicts.')
+                self.stdout.write(err)
+                self.json.add_error('The patch was partially applied, but '
+                                    'there were conflicts.')
 
             return False
 
         return True
 
+    def initialize(self):
+        """Initialize the command.
+
+        This overrides Command.initialize in order to handle full review
+        request URLs on the command line. In this case, we want to parse that
+        URL in order to pull the server name and diff revision out of it.
+
+        Raises:
+            rbtools.commands.CommandError:
+                A review request URL passed in as the review request ID could
+                not be parsed correctly or included a bad diff revision.
+        """
+        review_request_id = self.options.args[0]
+
+        if review_request_id.startswith('http'):
+            server_url, review_request_id, diff_revision = \
+                parse_review_request_url(review_request_id)
+
+            if diff_revision and '-' in diff_revision:
+                raise CommandError('Interdiff patches are not supported: %s.'
+                                   % diff_revision)
+
+            if review_request_id is None:
+                raise CommandError('The URL %s does not appear to be a '
+                                   'review request.')
+
+            self.options.server = server_url
+            self.options.diff_revision = diff_revision
+            self.options.args[0] = review_request_id
+
+        self.needs_scm_client = (not self.options.patch_stdout and
+                                 not self.options.patch_outfile)
+        self.needs_repository = self.needs_scm_client
+
+        super(Patch, self).initialize()
+
     def main(self, review_request_id):
         """Run the command.
 
@@ -409,9 +461,11 @@ class Patch(Command):
             rbtools.command.CommandError:
                 Patching the tree has failed.
         """
+        self._review_request_id = review_request_id
         patch_stdout = self.options.patch_stdout
         patch_outfile = self.options.patch_outfile
         revert = self.options.revert_patch
+        tool = self.tool
 
         if revert:
             if patch_stdout:
@@ -423,46 +477,19 @@ class Patch(Command):
                     _('--write and --revert cannot both be used.'))
 
         if patch_stdout and patch_outfile:
-                raise CommandError(
-                    _('--print and --write cannot both be used.'))
+            raise CommandError(
+                _('--print and --write cannot both be used.'))
 
-        repository_info, tool = self.initialize_scm_tool(
-            client_name=self.options.repository_type,
-            require_repository_info=(not patch_stdout and
-                                     not patch_outfile))
-
-        server_url = None
-        diff_revision = None
-
-        if review_request_id.startswith('http'):
-            (server_url,
-             review_request_id,
-             diff_revision) = parse_review_request_url(review_request_id)
-
-            if diff_revision and '-' in diff_revision:
-                raise CommandError('Interdiff patches not supported: %s.'
-                                   % diff_revision)
-
-        if server_url is None:
-            server_url = self.get_server_url(repository_info, tool)
-
-        if diff_revision is None:
-            diff_revision = self.options.diff_revision
+        if patch_stdout and self.options.json_output:
+            raise CommandError(
+                _('--print and --json cannot both be used.'))
 
         if revert and not tool.supports_patch_revert:
             raise CommandError(
                 _('The %s backend does not support reverting patches.')
                 % tool.name)
 
-        api_client, api_root = self.get_api(server_url)
-        self.setup_tool(tool, api_root=api_root)
-
         if not patch_stdout and not patch_outfile:
-            # Check if the repository info on the Review Board server matches
-            # the local checkout.
-            repository_info = repository_info.find_server_repository_info(
-                api_root)
-
             # Check if the working directory is clean.
             try:
                 if tool.has_pending_changes():
@@ -472,15 +499,10 @@ class Patch(Command):
                         raise CommandError(message)
                     else:
                         logger.warning(message)
+                        self.json.add_warning(message)
             except NotImplementedError:
                 pass
 
-        # Store the instances we've set up so that other commands have access.
-        self._api_root = api_root
-        self._repository_info = repository_info
-        self._tool = tool
-        self._review_request_id = review_request_id
-
         if self.options.commit_ids:
             # Do our best to normalize what gets passed in, so that we don't
             # end up with any blank entries.
@@ -496,18 +518,13 @@ class Patch(Command):
         # Fetch the patches from the review request, based on the requested
         # options.
         patches = self.get_patches(
-            diff_revision=diff_revision,
+            diff_revision=self.options.diff_revision,
             commit_ids=commit_ids,
             squashed=self.options.squash,
             reverted=revert)
 
         if patch_stdout:
-            if six.PY3:
-                stream = sys.stdout.buffer
-            else:
-                stream = sys.stdout
-
-            self._output_patches(patches, stream)
+            self._output_patches(patches, self.stdout_bytes)
         elif patch_outfile:
             try:
                 with open(patch_outfile, 'wb') as fp:
@@ -551,29 +568,29 @@ class Patch(Command):
         total_patches = len(patches)
 
         # Check if we're planning to commit and have any patch without
-        # metadata, in which case we'll need to fetch the review request so we
-        # can generate a commit message.
-        needs_review_request = will_commit and (
+        # metadata, in which case we'll need to fetch metadata from the
+        # review request so we can generate a commit message.
+        needs_review_request_metadata = will_commit and (
             squash or total_patches == 1 or
             any(patch_data['commit_meta'] is None for patch_data in patches)
         )
 
-        if needs_review_request:
-            # Fetch the review request to use as a description. We only
-            # want to fetch this once.
-            try:
-                review_request = self._api_root.get_review_request(
-                    review_request_id=self._review_request_id,
-                    force_text_type='plain')
-            except APIError as e:
-                raise CommandError(
-                    _('Error getting review request %(review_request_id)d: '
-                      '%(error)s')
-                    % {
-                        'review_request_id': self._review_request_id,
-                        'error': e,
-                    })
+        # Fetch the review request to use as a description and for URLs in
+        # JSON metadata. We only want to fetch this once.
+        try:
+            review_request = self.api_root.get_review_request(
+                review_request_id=self._review_request_id,
+                force_text_type='plain')
+        except APIError as e:
+            raise CommandError(
+                _('Error getting review request %(review_request_id)d: '
+                  '%(error)s')
+                % {
+                    'review_request_id': self._review_request_id,
+                    'error': e,
+                })
 
+        if needs_review_request_metadata:
             default_author = review_request.get_submitter()
             default_commit_message = extract_commit_message(review_request)
         else:
@@ -605,6 +622,10 @@ class Patch(Command):
                 'review_request_id': self._review_request_id,
                 'diff_revision': diff_revision,
             })
+        self.json.add('review_request_id', review_request.id)
+        self.json.add('review_request_url', review_request.absolute_url)
+        self.json.add('diff_revision', diff_revision)
+        self.json.add('total_patches', total_patches)
 
         # Start applying all the patches.
         for patch_data in patches:
@@ -626,6 +647,8 @@ class Patch(Command):
                 else:
                     error = _('Could not revert patch %(num)d of %(total)d')
 
+                self.json.add('failed_patch_num', patch_num)
+
                 raise CommandError(error % {
                     'num': patch_num,
                     'total': total_patches,
@@ -667,7 +690,7 @@ class Patch(Command):
                     message = '[Revert] %s' % message
 
                 try:
-                    self._tool.create_commit(
+                    self.tool.create_commit(
                         message=message,
                         author=author,
                         run_editor=not commit_no_edit)
@@ -675,4 +698,4 @@ class Patch(Command):
                     raise CommandError(six.text_type(e))
                 except NotImplementedError:
                     raise CommandError('--commit is not supported with %s'
-                                       % self._tool.name)
+                                       % self.tool.name)
